Imports System.Drawing.Printing

''' <summary>
''' Routinen fr die Druckersteuerung
''' </summary>
''' <remarks></remarks>
Public Class clsPrinter

    Private m_Doc As PrintDocument
    Private m_LeftMargin As Single
    Private m_TopMargin As Single
    Private m_MaxHeight As Single
    Private m_HardX As Single
    Private m_HardY As Single
    Private MMFaktor As Single = 0.254

    ''' <summary>
    ''' Linker Seitenrand
    ''' </summary>
    Public Property LeftMargin() As Single
        Get
            LeftMargin = m_LeftMargin
        End Get
        Set(ByVal value As Single)
            m_LeftMargin = value
        End Set
    End Property

    ''' <summary>
    ''' oberer Seitenrand
    ''' </summary>
    Public Property TopMargin() As Single
        Get
            TopMargin = m_TopMargin
        End Get
        Set(ByVal value As Single)
            m_TopMargin = value
        End Set
    End Property

    ''' <summary>
    ''' maximale Seitenhhe ab TopMargin
    ''' </summary>
    Public Property MaxHeight() As Single
        Get
            MaxHeight = m_MaxHeight
        End Get
        Set(ByVal value As Single)
            m_MaxHeight = value
        End Set
    End Property

    ''' <summary>
    ''' liefert den physikalischen Seitenrand links
    ''' </summary>
    Public ReadOnly Property HardX() As Single
        Get
            HardX = m_HardX
        End Get
    End Property

    ''' <summary>
    ''' liefert den physikalischen Seitenrand oben
    ''' </summary>
    Public Property HardY() As Single
        Get
            HardY = m_HardY
        End Get
        Set(ByVal value As Single)

        End Set
    End Property

    ''' <summary>
    ''' Initialize Class
    ''' </summary>
    Public Sub New(ByVal Doc As PrintDocument,
                   ByVal mLeftMargin As Single, _
                   ByVal mTopMargin As Single, _
                   ByVal mMaxHeight As Single)

        m_Doc = Doc
        'physikalische Druckrnder ermitteln
        m_HardX = Doc.DefaultPageSettings.HardMarginX * MMFaktor
        m_HardY = Doc.DefaultPageSettings.HardMarginY * MMFaktor
        m_LeftMargin = mLeftMargin
        m_TopMargin = mTopMargin
        m_MaxHeight = mMaxHeight
    End Sub

    ''' <summary>
    ''' Drucken String Linksbndig
    ''' </summary>
    Public Sub PrintLeft(ByVal s As String, ByVal Gr As Graphics, _
                         ByVal Ft As Font, ByVal Br As Brush, _
                         ByVal X As Single, ByVal Y As Single, _
                         Optional ByVal MaxWidth As Single = 0)

        Dim x1 As Single = LeftMargin + X - HardX
        Dim y1 As Single = TopMargin + Y - HardY

        If MaxWidth > 0 Then
            If Gr.MeasureString(s, Ft).Width > MaxWidth Then
                Do While (Gr.MeasureString(s & "..", Ft).Width > MaxWidth)
                    s = s.Substring(0, s.Length - 1)
                Loop
                s = s & ".."
            End If
        End If
        Gr.DrawString(s, Ft, Br, x1, y1)
    End Sub

    ''' <summary>
    ''' Drucken String Rechtsbndig
    ''' </summary>
    Public Sub PrintRight(ByVal s As String, ByVal Gr As Graphics, _
                          ByVal Ft As Font, ByVal Br As Brush, _
                          ByVal X As Single, ByVal Y As Single, _
                          Optional ByVal MaxWidth As Single = 0)

        Dim x1 As Single = LeftMargin + X - HardX - _
                           Gr.MeasureString(s, Ft).Width
        Dim y1 As Single = TopMargin + Y - HardY

        If MaxWidth > 0 Then
            If Gr.MeasureString(s, Ft).Width > MaxWidth Then
                Do While (Gr.MeasureString(s & "..", Ft).Width > MaxWidth)
                    s = s.Substring(1)
                Loop
                s = ".." & s
            End If
        End If
        Gr.DrawString(s, Ft, Br, x1, y1)
    End Sub

    ''' <summary>
    ''' Drucken String Zentriert
    ''' </summary>
    Public Sub PrintCenter(ByVal s As String, ByVal Gr As Graphics, _
                           ByVal Ft As Font, ByVal Br As Brush, _
                           ByVal X As Single, ByVal Y As Single, _
                           Optional ByVal MaxWidth As Single = 0)

        Dim x1 As Single = LeftMargin + X - HardX - _
                           (Gr.MeasureString(s, Ft).Width / 2)
        Dim y1 As Single = TopMargin + Y - HardY

        If MaxWidth > 0 Then
            If Gr.MeasureString(s, Ft).Width > MaxWidth Then
                Do While (Gr.MeasureString(s & "..", Ft).Width > MaxWidth)
                    s = s.Substring(0, s.Length - 1)
                Loop
                s = s & ".."
            End If
        End If
        Gr.DrawString(s, Ft, Br, x1, y1)
    End Sub

    Public Sub PrintRotate90(ByVal s As String, ByVal Gr As Graphics, _
                             ByVal Ft As Font, ByVal Br As Brush, _
                             ByVal X As Single, ByVal Y As Single, _
                             Optional ByVal MaxWidth As Single = 0)

        If MaxWidth > 0 Then
            If Gr.MeasureString(s, Ft).Width > MaxWidth Then
                Do While (Gr.MeasureString(s & "..", Ft).Width > MaxWidth)
                    s = s.Substring(0, s.Length - 1)
                Loop
                s = s & ".."
            End If
        End If

        Dim x1 As Single = LeftMargin + X - HardX + _
                           Gr.MeasureString(s, Ft).Height
        Dim y1 As Single = TopMargin + Y - HardY

        Gr.TranslateTransform(x1, y1)
        Gr.RotateTransform(90)
        Gr.DrawString(s, Ft, Br, 0, 0)

        Gr.RotateTransform(270)
        Gr.TranslateTransform(x1 * -1, y1 * -1)
    End Sub


    ''' <summary>
    ''' Drucken String um 180 Grad gespiegelt
    ''' </summary>
    Public Sub PrintRotate180(ByVal s As String, ByVal Gr As Graphics, _
                              ByVal Ft As Font, ByVal Br As Brush, _
                              ByVal X As Single, ByVal Y As Single, _
                              Optional ByVal MaxWidth As Single = 0)

        If MaxWidth > 0 Then
            If Gr.MeasureString(s, Ft).Width > MaxWidth Then
                Do While (Gr.MeasureString(s & "..", Ft).Width > MaxWidth)
                    s = s.Substring(0, s.Length - 1)
                Loop
                s = s & ".."
            End If
        End If

        Dim x1 As Single = LeftMargin + X - HardX + _
                           Gr.MeasureString(s, Ft).Width
        Dim y1 As Single = TopMargin + Y - HardY + _
                           Gr.MeasureString(s, Ft).Height

        Gr.TranslateTransform(x1, y1)
        Gr.RotateTransform(180)
        Gr.DrawString(s, Ft, Br, 0, 0)

        Gr.RotateTransform(180)
        Gr.TranslateTransform(-x1, -y1)
    End Sub

    Public Sub PrintRotate270(ByVal s As String, ByVal Gr As Graphics, _
                              ByVal Ft As Font, ByVal Br As Brush, _
                              ByVal X As Single, ByVal Y As Single, _
                              Optional ByVal MaxWidth As Single = 0)

        If MaxWidth > 0 Then
            If Gr.MeasureString(s, Ft).Width > MaxWidth Then
                Do While (Gr.MeasureString(s & "..", Ft).Width > MaxWidth)
                    s = s.Substring(0, s.Length - 1)
                Loop
                s = s & ".."
            End If
        End If

        Dim x1 As Single = LeftMargin + X - HardX
        Dim y1 As Single = TopMargin + Y - HardY + _
                           Gr.MeasureString(s, Ft).Width

        Gr.TranslateTransform(x1, y1)
        Gr.RotateTransform(270)
        Gr.DrawString(s, Ft, Br, 0, 0)

        Gr.RotateTransform(90)
        Gr.TranslateTransform(x1 * -1, y1 * -1)
    End Sub


    ''' <summary>
    ''' eine Line zeichnen
    ''' </summary>
    Public Sub DrawLine(ByVal Gr As Graphics, ByVal P As Pen, _
                        ByVal x1 As Single, ByVal y1 As Single, _
                        ByVal x2 As Single, ByVal y2 As Single)

        x1 = x1 + LeftMargin - HardX
        x2 = x2 + LeftMargin - HardX
        y1 = y1 + TopMargin - HardY
        y2 = y2 + TopMargin - HardY
        Gr.DrawLine(P, x1, y1, x2, y2)
    End Sub

    ''' <summary>
    ''' ein Rechteck zeichen
    ''' </summary>
    Public Sub DrawRectangle(ByVal Gr As Graphics, ByVal P As Pen, _
                             ByVal x As Single, ByVal y As Single, _
                             ByVal Width As Single, ByVal Height As Single)

        Dim GU As GraphicsUnit = Gr.PageUnit

        Gr.PageUnit = GraphicsUnit.Display

        Dim x1 As Single = x + LeftMargin - HardX
        Dim y1 As Single = y + TopMargin - HardY
        Dim x2 As Integer = Convert.ToInt32(x1 / MMFaktor)
        Dim y2 As Integer = Convert.ToInt32(y1 / MMFaktor)
        Dim w1 As Integer = Convert.ToInt32(Width / MMFaktor)
        Dim h1 As Integer = Convert.ToInt32(Height / MMFaktor)

        Dim Rect As New System.Drawing.Rectangle(x2, y2, w1, h1)
        Gr.DrawRectangle(P, Rect)
        Gr.PageUnit = GU
    End Sub

    ''' <summary>
    ''' eine gefllte Box zeichnen
    ''' </summary>
    Public Sub DrawBox(ByVal Gr As Graphics, ByVal x As Single, _
                       ByVal y As Single, ByVal Width As Single, _
                       ByVal Height As Single, ByVal BackColor As Brush)

        Dim x1 As Single = x + LeftMargin - HardX
        Dim y1 As Single = y + TopMargin - HardY
        Dim R As New RectangleF(x1, y1, Width, Height)

        Gr.FillRectangle(BackColor, R)
    End Sub

    Public Sub DrawBox(ByVal Gr As Graphics, ByVal x As Single,
                       ByVal y As Single, ByVal Width As Single, _
                       ByVal Height As Single, ByVal BackColor As Brush, _
                       ByVal BorderWidth As Single, ByVal BorderColor As Color)

        Dim x1 As Single = x + LeftMargin - HardX
        Dim y1 As Single = y + TopMargin - HardY
        Dim P As New Pen(BorderColor, BorderWidth)

        DrawBox(Gr, x, y, Width, Height, BackColor)

        Gr.DrawRectangle(P, x1, y1, Width, Height)
    End Sub

    Public Function PageHeight() As Single
        Return m_Doc.DefaultPageSettings.Bounds.Height * MMFaktor
    End Function

    Public Function PageWidth() As Single
        Return m_Doc.DefaultPageSettings.Bounds.Width * MMFaktor
    End Function
End Class